@@ -20,7 +20,7 @@ from pypdf import (
     Transformation,
 )
 from pypdf.annotations import Link
-from pypdf.errors import DeprecationError, PageSizeNotDefinedError, PyPdfError
+from pypdf.errors import DeprecationError, PageSizeNotDefinedError, PdfReadError, PyPdfError
 from pypdf.generic import (
     ArrayObject,
     ByteStringObject,
@@ -1804,8 +1804,8 @@ def test_watermark_rendering(tmp_path):
     name = "srcwatermark.pdf"
     page = PdfReader(BytesIO(get_data_from_url(url, name=name))).pages[0]
     writer = PdfWriter()
+    page = writer.add_page(page)
     page.merge_page(watermark, over=False)
-    writer.add_page(page)
 
     target_png_path = tmp_path / "target.png"
     url = "https://github.com/py-pdf/pypdf/assets/96178532/d5c72d0e-7047-4504-bbf6-bc591c80d7c0"
@@ -1846,8 +1846,8 @@ def test_watermarking_reportlab_rendering(tmp_path):
     watermark = PdfReader(watermark_path).pages[0]
 
     writer = PdfWriter()
+    base_page = writer.add_page(base_page)
     base_page.merge_page(watermark)
-    writer.add_page(base_page)
 
     target_png_path = RESOURCE_ROOT / "test_watermarking_reportlab_rendering.png"
     pdf_path = tmp_path / "out.pdf"
@@ -1884,7 +1884,7 @@ def test_da_missing_in_annot():
     ff = reader.get_fields()
     # check for autosize processing
     assert (
-        b"0 Tf"
+        b" 0 Tf"
         not in ff["PCN-1"].indirect_reference.get_object()["/AP"]["/N"].get_data()
     )
     f2 = writer.get_object(ff["PCN-2"].indirect_reference.idnum)
@@ -2406,7 +2406,7 @@ def test_selfont():
         b"Text_1" in writer.pages[0]["/Annots"][1].get_object()["/AP"]["/N"].get_data()
     )
     assert (
-        b"/F3 12 Tf"
+        b"/F3 12.0 Tf"
         in writer.pages[0]["/Annots"][2].get_object()["/AP"]["/N"].get_data()
     )
     assert (
@@ -2427,7 +2427,7 @@ def test_no_resource_for_14_std_fonts(caplog):
             writer.update_page_form_field_values(
                 p, {a["/T"]: "Brooks"}, auto_regenerate=False
             )
-    assert "Font dictionary for /Helvetica not found." in caplog.text
+    assert "Font dictionary for /Helvetica not found; defaulting to Helvetica." in caplog.text
 
 
 @pytest.mark.enable_socket
@@ -2438,8 +2438,8 @@ def test_field_box_upside_down():
     writer = PdfWriter(BytesIO(get_data_from_url(url, name=name)))
     writer.update_page_form_field_values(None, {"FreightTrainMiles": "0"})
     assert writer.pages[0]["/Annots"][13].get_object()["/AP"]["/N"].get_data() == (
-        b"q\n/Tx BMC \nq\n1 1 105.29520000000001 10.835000000000036 re\n"
-        b"W\nBT\n/Arial 8.0 Tf 0 g\n2 2.8350000000000364 Td\n(0) Tj\nET\n"
+        b"q\n/Tx BMC \nq\n2 1 102.29520000000001 9.835000000000036 re\n"
+        b"W\nBT\n/Helv 8.0 Tf 0 g\n2 3.0455000000000183 Td\n(0) Tj\nET\n"
         b"Q\nEMC\nQ\n"
     )
     box = writer.pages[0]["/Annots"][13].get_object()["/AP"]["/N"]["/BBox"]
@@ -2851,3 +2851,47 @@ def test_unterminated_object__with_incremental_writer():
     writer.write(fi)
     b = fi.getvalue()
     assert b[-39:] == b"\nendstream\nendobj\nstartxref\n1240\n%%EOF\n"
+
+
+def test_wrong_size_in_incremental_pdf(caplog):
+    source_data = RESOURCE_ROOT.joinpath("crazyones.pdf").read_bytes()
+    writer = PdfWriter(BytesIO(source_data), incremental=True)
+    writer._add_object(DictionaryObject())
+
+    incremental_data = BytesIO()
+    writer.write(incremental_data)
+    modified_data = incremental_data.getvalue().replace(b"/Size 25", b"/Size 2")
+
+    writer = PdfWriter(BytesIO(modified_data), incremental=False)
+    assert "Object count 19 exceeds defined trailer size 2" in caplog.text
+    assert len(writer._objects) == 20
+
+    caplog.clear()
+    writer = PdfWriter(incremental=False, strict=True)
+    with pytest.raises(expected_exception=PdfReadError, match=r"^Object count 19 exceeds defined trailer size 2$"):
+        writer.clone_reader_document_root(reader=PdfReader(BytesIO(modified_data)))
+
+    with pytest.raises(expected_exception=PdfReadError, match=r"^Got index error while flattening\.$"):
+        PdfWriter(BytesIO(modified_data), incremental=True)
+
+
+@pytest.mark.enable_socket
+def test_flatten_form_field_without_font_in_resources():
+    """
+    This test is a regression test for issue #3553.
+    Flatten form field with /Resources lacking /Font.
+    """
+    reader = PdfReader(BytesIO(get_data_from_url(name="issue-3553.pdf")))
+    writer = PdfWriter()
+    writer.append(reader)
+    writer.update_page_form_field_values(
+        writer.pages[0],
+        {"Unique reference numberRow1": "test"},
+        flatten=True,
+    )
+    b = BytesIO()
+    writer.write(b)
+
+    reader = PdfReader(b)
+    form_text_fields = reader.get_form_text_fields()
+    assert form_text_fields["Unique reference numberRow1"] == "test"
